#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
test with:
    python setup.py build_ext --inplace

build wheels with:
    python setup.py bdist_wheel --plat-name win_amd64 --python-tag cp36
    python setup.py bdist_wheel --plat-name macosx_10_10_x86_64 --python-tag cp36
    python setup.py sdist --formats=gztar
    twine upload dist/*

https://pypi.python.org/pypi/ssw-py
'''
DESCRIPTION = ("Complete Striped Smith-Waterman Library for Python")
LONG_DESCRIPTION = '''
**ssw-py** is a Python package
Cythonized wrapped version of:

https://github.com/mengyao/Complete-Striped-Smith-Waterman-Library

Original C library authors and paper should be cited if used.

Support for Python 3 at the moment

License is MIT
'''

try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension

import numpy.distutils.misc_util
import os
import sys
import shutil
import re
import ast

pjoin = os.path.join
rpath = os.path.relpath

PACKAGE_PATH =      os.path.abspath(os.path.dirname(__file__))
MODULE_PATH =       pjoin(PACKAGE_PATH, 'ssw')
LIB_PATH =          pjoin(MODULE_PATH, 'lib')

common_include = ['ssw/lib/CSSWL/src', 'ssw/lib']

if sys.platform == 'win32':
    extra_compile_args = ['']
else:
    extra_compile_args = ['-Wno-unused-function']

# source files to include in installation for tar.gz
ssw_files = [rpath(pjoin(root, f), MODULE_PATH) for root, _, files in
                 os.walk(LIB_PATH) for f in files if (
                    ('.h' in f) or ('.c' in f) or ('.cpp' in f)
                )
]

ssw_ext = Extension(
    'ssw.sswpy',
    sources=['ssw/sswpy.pyx',
             'ssw/lib/CSSWL/src/ssw.c',
             'ssw/lib/str_util.c'],
    include_dirs=common_include + [numpy.get_include()],
    extra_compile_args=extra_compile_args
)

# Initialize subtree with
# git subtree add --prefix=lib/CSSWL git@github.com:mengyao/Complete-Striped-Smith-Waterman-Library.git master

# Begin modified code from Flask's version getter
# BSD license
# Copyright (c) 2015 by Armin Ronacher and contributors.
# https://github.com/pallets/flask
_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('ssw/__init__.py', 'rb') as initfile:
    VERSION = str(ast.literal_eval(_version_re.search(
                                   initfile.read().decode('utf-8')).group(1)))

DISTNAME = 'ssw-py'
LICENSE = 'MIT'
AUTHORS = "Nick Conway"
EMAIL = "nick.conway@wyss.harvard.edu"
CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Intended Audience :: Science/Research',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Cython',
    'Topic :: Scientific/Engineering',
]

setup(
    name=DISTNAME,
    version=VERSION,
    author=AUTHORS,
    author_email=EMAIL,
    url='https://github.com/Wyss/ssw-py',
    packages=['ssw'],
    ext_modules=[ssw_ext],
    include_dirs=numpy.distutils.misc_util.get_numpy_include_dirs(),
    package_data={'ssw': ssw_files},
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    license=LICENSE,
    classifiers=CLASSIFIERS,
    zip_safe=False
)



# -*- coding: utf-8 -*-
from typing import Union

from ssw import (
    SSW,
    Alignment
)

STR_T = Union[str, bytes]

def printer(alignment: Alignment, reference: STR_T, read: STR_T):
    print("optimal_score: %d\tsub-optimal_score: %d\t" %
        (alignment.optimal_score, alignment.sub_optimal_score))

    reference_start: int = alignment.reference_start

    if reference_start >= 0:
        print("reference_start: %d\t" % (reference_start), end='')
    print("reference_end: %d\t" % (alignment.reference_end) )

    read_start: int = alignment.read_start

    if read_start >= 0:
        print("read_start: %d\t" % (read_start), end='')
    print("read_end: %d\n" % (alignment.read_end))

    cigar: str = alignment.CIGAR

    if cigar is not None:
        e: int      = 0
        left: int   = 0
        pb: int = read_start
        qb: int = reference_start
        STEP2: bool = False
        STEP3: bool = False
        STEPEND: bool = False
        lim: int = len(cigar)

        while e < lim or left > 0:
            q: int = qb
            p: int = pb
            count: int = 0

            print("Refer:  %8d    " % (q), end='')
            for c in range(e, lim, 2):
                length = int(cigar[c])
                letter = cigar[c + 1]
                L1 = left if (count == 0 and left > 0) else length
                for j in range(L1):
                    if letter == 'I':
                        print("-", end='')
                    else:
                        print(reference[q], end='')
                        q += 1
                    count += 1
                    if count == 60:
                        STEP2 = True
                        break # goto STEP 2
                if STEP2:
                    STEP2 = False
                    break
            # end for
            # STEP 2
            print("    %d\n                    " % (q - 1), end='')
            q = qb
            count = 0
            for c in range(e, lim, 2):
                length = int(cigar[c])
                letter = cigar[c + 1]
                L1 = left if (count == 0 and left > 0) else length
                for j in range(L1):
                    if letter == 'M':
                        if reference[q] == read[p]:
                            print('|', end='')
                        else:
                            print("*", end='')
                        q += 1
                        p += 1
                    else:
                        print('*', end='')
                        if letter == 'I':
                            p += 1
                        else:
                            q += 1
                    count += 1
                    if count == 60:
                        qb = q
                        STEP3 = True
                        break # goto STEP 3
                if STEP3:
                    STEP3 = False
                    break
            # end for
            # STEP 3
            p = pb
            print("\nRead:   %8d    " % (p), end='')
            count = 0
            for c in range(e, lim, 2):
                length = int(cigar[c])
                letter = cigar[c + 1]
                L1 = left if (count == 0 and left > 0) else length
                for j in range(L1):
                    if letter == 'D':
                        print('-', end='')
                    else:
                        print(read[p], end='')
                        p += 1
                    count += 1
                    if count == 60:
                        pb = p
                        left = L1 - j - 1
                        e = c + 2 if left == 0 else c
                        STEPEND = True
                        break # goto STEPEND
                if STEPEND:
                    STEPEND = False
                    break
            e = c + 2
            left = 0
            # STEPEND
            print("    %d\n" % (p - 1))
        # end while
    #end if
# end def